@@ -136,7 +136,11 @@ h5createGroup <- function(file, group) {
       }
     )
   } else {
-    tid <- h5checkConstants("H5T", H5type)
+    if (grepl(pattern = "^[[:digit:]]+$", H5type)) {
+      tid <- H5type
+    } else {
+      tid <- h5checkConstants("H5T", H5type)
+    }
   }
   if (is.na(tid)) {
     stop(
@@ -346,9 +350,10 @@ h5createGroup <- function(file, group) {
 #' @param storage.mode The storage mode of the data to be written. Can be
 #'   obtained by `storage.mode(mydata)`.
 #' @param H5type Advanced programmers can specify the datatype of the dataset
-#'   within the file. See `h5const("H5T")` for a list of available
-#'   datatypes. If `H5type` is specified the argument `storage.mode`
-#'   is ignored. It is recommended to use `storage.mode`
+#'   within the file, either as a string with one of the available datatypes
+#'   listed in `h5const("H5T")`, or as the output of [H5Tcopy()]. If `H5type`
+#'   is specified the argument `storage.mode` is ignored. It is recommended
+#'   to use `storage.mode`.
 #' @param size For `storage.mode='character'` the maximum string length to use.
 #'   The default value of `NULL` will result in using variable length strings.
 #'   See the details for more information on this option.
@@ -448,7 +453,6 @@ h5createGroup <- function(file, group) {
 #' file.size(f1)
 #' file.size(f2)
 #'
-#' @name h5_createDataset
 #' @export h5createDataset
 h5createDataset <- function(
   file,
@@ -469,6 +473,7 @@ h5createDataset <- function(
   loc <- h5checktypeOrOpenLoc(file, native = native)
   on.exit(h5closeitLoc(loc))
 
+  encoding <- match.arg(encoding, choices = c("ASCII", "UTF-8", "UTF8"))
   dims <- as.numeric(dims)
   maxdims <- as.numeric(maxdims)
 
@@ -503,7 +508,7 @@ h5createDataset <- function(
     H5type,
     storage.mode,
     size,
-    encoding = match.arg(encoding, choices = c("ASCII", "UTF-8", "UTF8"))
+    encoding = encoding
   )
 
   dcpl <- .createDCPL(
@@ -559,12 +564,6 @@ h5createDataset <- function(
 #' @param dims The dimensions of the attribute as a numeric vector. If
 #'   `NULL`, a scalar dataspace will be created instead.
 #' @param maxdims The maximum extension of the attribute.
-#' @param storage.mode The storage mode of the data to be written. Can be
-#'   obtained by `storage.mode(mydata)`.
-#' @param H5type Advanced programmers can specify the datatype of the dataset
-#'   within the file. See `h5const("H5T")` for a list of available
-#'   datatypes. If `H5type` is specified the argument `storage.mode`
-#'   is ignored. It is recommended to use `storage.mode`
 #' @param size The maximum string length when `storage.mode='character'`.
 #'   If this is specified, HDF5 stores each string of `attr` as fixed
 #'   length character arrays. Together with compression, this should be
@@ -572,13 +571,8 @@ h5createDataset <- function(
 #'
 #'   If this argument is set to `NULL`, HDF5 will instead store
 #'   variable-length strings.
-#' @param encoding The encoding of the string data type i.e. when `storage.mode
-#'   = 'character'`. Valid options are "ASCII" and "UTF-8".
-#' @param native An object of class `logical`. If TRUE, array-like objects
-#'   are treated as stored in HDF5 row-major rather than R column-major
-#'   orientation. Using `native = TRUE` increases HDF5 file portability
-#'   between programming languages. A file written with `native = TRUE`
-#'   should also be read with `native = TRUE`
+#' @inheritParams h5createDataset
+#'
 #' @returns Returns TRUE is attribute was created successfully and FALSE
 #'   otherwise.
 #' @author Bernd Fischer
@@ -597,7 +591,6 @@ h5createDataset <- function(
 #' H5Dclose(did)
 #' H5Fclose(fid)
 #'
-#' @name h5_createAttribute
 #' @export h5createAttribute
 h5createAttribute <- function(
   obj,
@@ -614,6 +607,7 @@ h5createAttribute <- function(
   obj <- h5checktypeOrOpenObj(obj, file, native = native)
   on.exit(h5closeitObj(obj))
 
+  encoding <- match.arg(encoding, choices = c("ASCII", "UTF-8", "UTF8"))
   res <- FALSE
 
   if (is.null(dims)) {
@@ -628,6 +622,11 @@ h5createAttribute <- function(
   }
   on.exit(H5Sclose(sid), add = TRUE)
 
+  # FIXME: Ultimately, this should be replaced by a call to .setDataType()
+  # but this would be a breaking change as .setDataType() and the code here
+  # treat logical differently.
+  # See https://github.com/Huber-group-EMBL/rhdf5/issues/162 for some
+  # discussion on this.
   if (is.null(H5type)) {
     if (!is.character(storage.mode)) {
       stop("Can not create dataset. 'storage.mode' has to be a character.")
@@ -640,7 +639,7 @@ h5createAttribute <- function(
         tid <- H5Tcopy("H5T_C_S1")
         H5Tset_cset(
           tid,
-          cset = match.arg(encoding, choices = c("ASCII", "UTF-8", "UTF8"))
+          cset = encoding
         )
         if (!is.null(size) && !is.numeric(size)) {
           stop(
@@ -669,9 +668,11 @@ h5createAttribute <- function(
   } else {
     if (grepl(pattern = "^[[:digit:]]+$", H5type)) {
       tid <- H5type
+    } else {
+      tid <- h5checkConstants("H5T", H5type)
     }
   }
-  if (!grepl(pattern = "^[[:digit:]]+$", tid)) {
+  if (is.na(tid)) {
     message(
       "Can not create attribute. H5type unknown. Check h5const('H5T') for valid types."
     )
