@@ -14,19 +14,15 @@ test_that("Tests for filters work", {
 })
 
 test_that("Missing filters are identified", {
-  skip_if_not_installed("mockery")
-
   dcpl <- H5Pcreate(type = "H5P_DATASET_CREATE")
 
   ## lots of things to mock here!
-  mockery::stub(where = h5checkFilters, what = "H5Pget_nfilters", how = 1)
-  mockery::stub(where = h5checkFilters, what = "H5Pall_filters_avail", how = 0)
-  mockery::stub(
-    where = h5checkFilters,
-    what = "H5Pget_filter",
-    how = list(404, "foo_filter")
+  local_mocked_bindings(
+    H5Pget_nfilters = function(h5plist) 1,
+    H5Pall_filters_avail = function(h5plist) 0,
+    H5Pget_filter = function(h5plist, index) list(404, "foo_filter"),
+    H5Zfilter_avail = function(filter_id) FALSE
   )
-  mockery::stub(where = h5checkFilters, what = "H5Zfilter_avail", how = FALSE)
 
   expect_true(grepl(
     pattern = "Missing filters: foo_filter",
